/* MTH02 library
*/

#include "mth02.h"

#define MIN_INTERVAL 2000

MTH::MTH(byte pin) {
	_pin = pin;	
}

void MTH::begin(void) {
	// set up the pins!
	pinMode(_pin, OUTPUT);
	digitalWrite(_pin, HIGH);
	// Using this value makes sure that millis() - lastreadtime will be
	// >= MIN_INTERVAL right away. Note that this assignment wraps around,
	// but so will the subtraction.
	_lastreadtime = -MIN_INTERVAL;	
}

//boolean S == Scale.  True == Fahrenheit; False == Celcius
float MTH::readTemperature(bool S) {
	float temp = 0;
	if (getDataBits()){
		temp = data[2] << 8 | data[3];
		if (temp >= 400) {
			temp = (temp - 400) / 10;
		}
		else {
			temp = (400 - temp) / 10;
		}
	}
	if (S){
		temp = convertCtoF(temp);
	}
	return temp;
}

float MTH::convertCtoF(float c) {
	return c * 1.8 + 32;
}


float MTH::readHumidity() {
	float hum = 0.0;
	if (getDataBits()){
		hum = data[0] << 8 | data[1];
		hum = hum / 10;
	}
	return hum;
}

//Get Data bits
boolean MTH::getDataBits() {
	int buf[40];	
	int currenttime = millis();

	if ((currenttime - _lastreadtime) < MIN_INTERVAL) {
		return _lastresult; // return last correct measurement
	}
	_lastreadtime = currenttime;

	data[0] = data[1] = data[2] = data[3] = data[4] = 0;

	//Get pulses
	digitalWrite(_pin, LOW);
	delay(100);
	pinMode(_pin, INPUT);

	noInterrupts(); //No need to interupt, while getting pulses into buffer  
	for (int i = 0; i < 40; i++){
		buf[i] = pulseIn(_pin, HIGH); //Insert lenght of the HIGH pulses into array		
		if (buf[i] == 0){
			DEBUG_PRINTLN(F("HIGH pulse is empty!"));
			_lastresult = false;
			return _lastresult;
		}
	}
	interrupts();

	delay(100);
	pinMode(_pin, OUTPUT);
	digitalWrite(_pin, HIGH);

	
	for (int i = 0; i < 40; i++){ //Read buffer and check for pulses lenght
		data[i / 8] <<= 1; //Move left one bit
		if (buf[i] > 50){ //If longer than LOW pulse, then - it's 1       
			data[i / 8] |= 1;
		}		
	}

	DEBUG_PRINTLN(F("Received:"));
	DEBUG_PRINT(data[0], HEX); DEBUG_PRINT(F(", "));
	DEBUG_PRINT(data[1], HEX); DEBUG_PRINT(F(", "));
	DEBUG_PRINT(data[2], HEX); DEBUG_PRINT(F(", "));
	DEBUG_PRINT(data[3], HEX); DEBUG_PRINT(F(", "));
	DEBUG_PRINT(data[4], HEX); DEBUG_PRINT(F(" =? "));
	DEBUG_PRINTLN((data[0] + data[1] + data[2] + data[3]) & 0xFF, HEX);

	// Check we read 40 bits and that the checksum matches.	
	if (data[4] == ((data[0] + data[1] + data[2] + data[3]) & 0xFF)) {
		_lastresult = true;
		return _lastresult;
	}
	else {
		DEBUG_PRINTLN(F("Checksum failure!"));
		_lastresult = false;
		return _lastresult;
	}

}
//Get data bits